{$B-}
unit AudioInfoDemoMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, StdCtrls, ComCtrls, ExtCtrls,
  AudioInfo, Spin;

type
  TfmAudioInfoDemo = class(TForm)
    pnlToolBar: TPanel;
    pnlBottom: TPanel;
    pnlBottomRight: TPanel;
    btnExit: TBitBtn;
    pgAudioInfo: TPageControl;
    tabMp3Header: TTabSheet;
    lblFileName: TLabel;
    pnlFileName: TPanel;
    pnlFileNameBtn: TPanel;
    edFileName: TEdit;
    btnFileName: TSpeedButton;
    OpenDlg: TOpenDialog;
    tabID3: TTabSheet;
    lblArtist: TLabel;
    edArtist: TEdit;
    lblTitle: TLabel;
    edTitle: TEdit;
    lblAlbum: TLabel;
    edAlbum: TEdit;
    lblYear: TLabel;
    edYear: TEdit;
    lblGenre: TLabel;
    edGenre: TComboBox;
    lblComment: TLabel;
    edComment: TEdit;
    btnID3Update: TSpeedButton;
    btnID3Kill: TSpeedButton;
    lblTrack: TLabel;
    edTrack: TSpinEdit;
    lblID3Status: TLabel;
    lblDurationHdr: TLabel;
    lblDuration: TLabel;
    lblMpegVersion: TLabel;
    lblMpegFlags: TLabel;
    lblMPegParam: TLabel;
    lblMpegInternal: TLabel;
    lblVBR: TLabel;
    procedure btnExitClick(Sender: TObject);
    procedure btnFileNameClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnID3UpdateClick(Sender: TObject);
    procedure btnID3KillClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
   private
    a : TAudioInfo;
    procedure ShowAudioInfo;
    function ChangeFile : boolean;
   public
  end;

var
  fmAudioInfoDemo: TfmAudioInfoDemo;

implementation
{$R *.DFM}

uses
 ID3Tags;

procedure TfmAudioInfoDemo.ShowAudioInfo;
 procedure SelectGenre (AID : integer);
  begin
   edGenre.ItemIndex := edGenre.Items.IndexOfObject (
                          TObject (AID));
  end;
 var
  s : string;
 begin
  if a.ID3.Ok then begin
     s := 'Found ID3v1';
     if not a.ID3.v1
      then s := s + '.1';
     s := s + ' tag';
     lblID3Status.Caption := s;

     edArtist.Text := a.ID3.Artist;
     edTitle.Text := a.ID3.Title;
     SelectGenre (a.ID3.GenreID);
     edYear.Text := a.ID3.Year;
     edComment.Text := a.ID3.Comment;
     edAlbum.Text := a.ID3.Album;
     edTrack.Value := a.ID3.Track;
    end
   else begin
     lblID3Status.Caption := 'ID3v1 tag not found. You may add new one';
     edArtist.Text := '';
     edTitle.Text := '';
     SelectGenre (255);
     edYear.Text := '';
     edComment.Text := '';
     edAlbum.Text := '';
     edTrack.Value := 0;
    end;
  if a.Mpg.Ok then begin
     lblVBR.Visible := a.XingVBR;
     lblDuration.Caption :=
      FormatDateTime ('h:nn:ss', a.MpegDuration / (24.0 * 60 * 60 * 1000))
      + Format (' (%d seconds)', [a.MpegDuration div 1000]);
     lblMpegVersion.Caption :=
       'MPEG ' + a.Mpg.VersionStr + ' layer ' + a.Mpg.LayerStr;
     if a.Mpg.Sync then s := 'Sync'
      else s := 'No sync';
     if a.Mpg.CRCExists then s := s + ', CRC'
      else s := s + ', No CRC';
     if a.Mpg.Copyright then s := s + ', Copyright'
      else s := s + ', No Copyright';
     if a.Mpg.Original then s := s + ', Original'
      else s := s + ', No Original';
     lblMpegFlags.Caption := '(' + s + ')';
     lblMpegParam.Caption := IntToStr (a.Mpg.KBitRate) + ' kbps, '
      + IntToStr (a.Mpg.SamplingRate) + ' Hz, '
      + a.Mpg.ChannelModeStr + ' ' + a.Mpg.ChannelModeExtStr + ' ';
     lblMpegInternal.Caption := 'Frame len=' + IntToStr (a.Mpg.FrameLen)
      + ', ' + IntToStr (a.FileSize div a.Mpg.FrameLen) + ' frames';

    end
   else begin
     lblVBR.Visible := false;
     lblDuration.Caption := '???';
     lblMpegVersion.Caption := '???';
     lblMpegFlags.Caption := '???';
     lblMpegParam.Caption := '???';
     lblMpegInternal.Caption := '???';
    end;
 end;

function TfmAudioInfoDemo.ChangeFile : boolean;
 begin
  Result := false;
  if OpenDlg.Execute then begin
    edFileName.Text := OpenDlg.FileName;
    a.LoadFromFile (OpenDlg.FileName);
    Result := true;
    ShowAudioInfo;
   end;
 end;

procedure TfmAudioInfoDemo.FormCreate(Sender: TObject);
 begin
  a := TAudioInfo.Create;
  ID3Genres.Sorted := true;
  edGenre.Items.Assign (ID3Genres);
 end;

procedure TfmAudioInfoDemo.btnExitClick(Sender: TObject);
 begin
  Close;
 end;

procedure TfmAudioInfoDemo.btnFileNameClick(Sender: TObject);
 begin
  ChangeFile;
 end;


procedure TfmAudioInfoDemo.btnID3UpdateClick(Sender: TObject);
 begin
  if edFileName.Text <> '' then
    if MessageBox (0, 'Save ID3 tag into file ?',
                   'ID3 tag update', MB_YESNO) = IDYES then begin
    a.ID3.Artist := edArtist.Text;
    a.ID3.Title := edTitle.Text;
    a.ID3.GenreID := integer (edGenre.Items.Objects [edGenre.ItemIndex]);
    a.ID3.Year := edYear.Text;
    a.ID3.Comment := edComment.Text;
    a.ID3.Album := edAlbum.Text;
    a.ID3.Track := edTrack.Value;
    a.ID3.SaveToFile (edFileName.Text);

    a.ID3.LoadFromFile (edFileName.Text);
    ShowAudioInfo;
   end;
 end;

procedure TfmAudioInfoDemo.btnID3KillClick(Sender: TObject);
 begin
  if edFileName.Text <> '' then
    if MessageBox (0, 'Delete ID3 tag from file ?',
                   'ID3 tag DELETE !', MB_YESNO) = IDYES then begin
      a.ID3.DeleteFromFile (edFileName.Text);
      a.ID3.LoadFromFile (edFileName.Text);
      ShowAudioInfo;
     end;
 end;

procedure TfmAudioInfoDemo.FormShow(Sender: TObject);
 begin
  if edFileName.Text = '' then begin
    if not ChangeFile
     then Close;
   end;
 end;

end.
